%% Backwardsimulation used for the pseudo-sequential MCS
% Written by Trygve V. Berg as part of a master thesis

%% Backward simulation function
function [durationbackw,outagecounterbackw,Plossbackw,EENSbackw] = backwardsim(linedata,L,outagebranches,mpc,dp_buses,mpopt,corrective_actions_data,contanalysis_settings)
    
    %Indices to be calculated in the backward simulation
    durationbackw = zeros(length(dp_buses),1);
    outagecounterbackw = zeros(length(dp_buses),1);
    Plossbackw = zeros(length(dp_buses),1);
    EENSbackw = zeros(length(dp_buses),1);
    
    outagebranchesbackw = outagebranches;
    previousstate = zeros(L,1);
    linedata(:,7) = linedata(:,8); %update current state column with elements of next state column to set the starting point for the forward simulation
    transition = zeros(1,L); %transition rate vector
    
    while sum(linedata(:,7)) > 0 % as long as there are failed states do the forward simulation
        for m = 1:L
            if linedata(m,7) == 1 %if current state is failure
                transition(m) = linedata(m,6); %transition rate is repair rate
            else
                transition(m) = linedata(m,5); %if no failure, transition rate is fault rate
            end %if linedata..
        end %for m
        
        transitionsum = sum(transition); %sum of transition rates
        probnextstate = zeros(1,L);
        cumulativeprob = zeros(1,L);
        for j = 1:L
            probnextstate(j) = transition(j)/transitionsum; %vector with probabilities for next state
            for l = 1:j
                cumulativeprob(j) = cumulativeprob(j) + probnextstate(l); %cumulative probability vector
            end %for l
        end %for j
        
        R = rand(); %drawing a random uniform number to decide next state (checking with cumulativeprob)
        if R < cumulativeprob(1) % If U < cumulativeprob(1) then histcounts does not work
            comp = 1;
        else
            comp = find(histcounts(R,cumulativeprob))+1; %finding which interval U falls under
        end % if R
            linedata(comp,8) = switchstate(linedata(comp,8)); %changing the next state of the component
            durationnextstate = (1/transition(comp))/8736; %finding the duration until next state
            durationnextstatedp = zeros(length(dp_buses),1); 
            outagecounter = zeros(length(dp_buses),1);
            Ploss = contanalysis_singlecont_montecarlosim(outagebranchesbackw,mpc,dp_buses,mpopt,corrective_actions_data,contanalysis_settings); %calculating Ploss until next state 
                                 
                for i = 1:length(Ploss) % checking for which dp the duration and frequency happens for
                if Ploss(i) > 0 %Not an elegant solution, but should work
                        if sum(linedata(:,7)) > sum(previousstate(:)) %checking if a new failure is introduced
                            outagecounter(i) = outagecounter(i) + 1; %adding the new failure to the number of failures for that component
                        end %if sum(linedata)...
                        durationnextstatedp(i) = durationnextstate; % accumulating interruption duration for specific year (+1 hour for each contingency)
                end % if Ploss(i)
            end %for i
        
        durationbackw = durationbackw + durationnextstatedp; %summing up forward simulation duration
        Plossbackw = Plossbackw + Ploss; %summing up forward sim Ploss
        EENSbackw = EENSbackw + Ploss.*durationnextstatedp; %summing up EENS
        outagecounterbackw = outagecounterbackw + outagecounter; %summing up number of outages (frequency)
        
        for o = 1:L
            if linedata(o,7) == 1 && linedata(o,8) == 0 %if changing from failure to working
                outagebranchesbackw(outagebranchesbackw == o) = []; %remove outage branch from outage list
            elseif linedata(o,7) == 0 && linedata(o,8) == 1 %if changing from working to failure
                outagebranchesbackw = [outagebranchesbackw, o]; %add outage branch to the end of outage list
                outagebranchesbackw = sort(outagebranchesbackw); %sorting outage list to be in ascending order
            end %if linedata
        end %for o
        previousstate(:) = linedata(:,7); %update previous state column vector (for keeping track of frequency index)
        linedata(:,7) = linedata(:,8); %update current state column with next state column, to keep iterating
    
    end %while sum(linedata)...
end %function